import os
import ImportGui
import FreeCADGui as Gui
import FreeCAD as App
import Materials
import Part

from kipy import KiCad
from kipy.board_types import Footprint3DModel, BoardPolygon, BoardSegment, PadStackShape
from kipy.util.board_layer import BoardLayer

from . import settings
from .bases import BaseObject, BaseViewProvider

class PartsObject(BaseObject):
    pass

class PartsViewProvider(BaseViewProvider):
    pass

class Parts():
    ICON = 'icon_footprint_browser.svg'
    TYPE = 'KiConnect::Parts'

    def __init__(self, kicad_board, kiconn_board):
        self.kicad_board = kicad_board
        self.kiconn_board = kiconn_board

        feature = App.ActiveDocument.addObject('App::DocumentObjectGroupPython', 'Parts')

        PartsObject(self, feature)
        PartsViewProvider(self, feature.ViewObject)

        self.feature = feature

        self.import_footprints()

    def import_footprints(self):
        for footprint in self.kicad_board.get_footprints():
            # NOTE this doesn't handle footprints that have been removed
            if App.ActiveDocument.getObjectsByLabel(footprint.reference_field.text.value): continue

            for item in [item for item in footprint.definition.items if isinstance(item, Footprint3DModel)]:
                filename = item.filename.replace('${KICAD9_3DMODEL_DIR}', settings.KICAD9_3DMODEL_DIR).replace('wrl', 'step')
                ImportGui.insert(filename, App.ActiveDocument.Name)

                # simply grabs the last object in the document, probably need to figure out a safer way to handle
                model = App.ActiveDocument.findObjects()[-1]
                model.Label = footprint.reference_field.text.value

                model.addProperty('App::PropertyPlacement', 'BoardOffset', 'Base', 'Internal offset for zeroing out Footprint offset', hidden=True, read_only=True)
                
                self.feature.addObject(model)

                model.Placement.Base.x = (footprint.position.x / 1000000.0) - self.kiconn_board.offset.x
                model.Placement.Base.y = (-footprint.position.y / 1000000.0) - self.kiconn_board.offset.y
                model.Placement.Base.z = 0.8
                model.Placement.Rotation.Angle = footprint.orientation.to_radians()
                model.BoardOffset = model.Placement

